// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Copyright 2017 and onwards Google, Inc.

#ifndef BAUMWELCH_TRAINSCRIPT_H_
#define BAUMWELCH_TRAINSCRIPT_H_

#include <fst/extensions/far/far-class.h>
#include <fst/script/arg-packs.h>
#include <fst/script/fst-class.h>
#include <baumwelch/train.h>

namespace fst {
namespace script {

bool GetExpectationTableType(const string &str, ExpectationTableType *etype);

// 1: Pair construction.
using TrainBaumWelchInnerArgs1 =
    args::Package<FarReaderClass *, FarReaderClass *, MutableFstClass *,
                  ExpectationTableType, const BaumWelchTrainOptions &>;

using TrainBaumWelchArgs1 =
    args::WithReturnValue<bool, TrainBaumWelchInnerArgs1>;

template <class Arc>
void TrainBaumWelch(TrainBaumWelchArgs1 *args) {
  FarReader<Arc> *plaintext = args->args.arg1->GetFarReader<Arc>();
  FarReader<Arc> *ciphertext = args->args.arg2->GetFarReader<Arc>();
  MutableFst<Arc> *channel = args->args.arg3->GetMutableFst<Arc>();
  args->retval = TrainBaumWelch(plaintext, ciphertext, channel, args->args.arg4,
                                args->args.arg5);
}

// 2: Decipherment construction.
using TrainBaumWelchInnerArgs2 =
    args::Package<const FstClass &, FarReaderClass *, MutableFstClass *,
                  ExpectationTableType, const BaumWelchTrainOptions &>;

using TrainBaumWelchArgs2 =
    args::WithReturnValue<bool, TrainBaumWelchInnerArgs2>;

template <class Arc>
void TrainBaumWelch(TrainBaumWelchArgs2 *args) {
  const Fst<Arc> &plaintext = *(args->args.arg1.GetFst<Arc>());
  FarReader<Arc> *ciphertext = args->args.arg2->GetFarReader<Arc>();
  MutableFst<Arc> *channel = args->args.arg3->GetMutableFst<Arc>();
  args->retval = TrainBaumWelch(plaintext, ciphertext, channel, args->args.arg4,
                                args->args.arg5);
}

bool TrainBaumWelch(
    FarReaderClass *plaintext, FarReaderClass *ciphertext,
    MutableFstClass *channel, ExpectationTableType etype = STATE_ILABEL,
    const BaumWelchTrainOptions &opts = BaumWelchTrainOptions());

bool TrainBaumWelch(
    const FstClass &plaintext, FarReaderClass *ciphertext,
    MutableFstClass *channel, ExpectationTableType etype = STATE_ILABEL,
    const BaumWelchTrainOptions &opts = BaumWelchTrainOptions());

}  // namespace script
}  // namespace fst

#endif  // BAUMWELCH_TRAINSCRIPT_H_

